/* valaccodegotostatement.vala
 *
 * Copyright (C) 2007  Jürg Billeter
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 */

#include <ccode/valaccodegotostatement.h>
#include <ccode/valaccodelinedirective.h>

struct _ValaCCodeGotoStatementPrivate {
	char* _name;
};
#define VALA_CCODE_GOTO_STATEMENT_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_CCODE_GOTO_STATEMENT, ValaCCodeGotoStatementPrivate))
enum  {
	VALA_CCODE_GOTO_STATEMENT_DUMMY_PROPERTY,
	VALA_CCODE_GOTO_STATEMENT_NAME
};
static void vala_ccode_goto_statement_real_write (ValaCCodeNode* base, ValaCCodeWriter* writer);
static gpointer vala_ccode_goto_statement_parent_class = NULL;
static void vala_ccode_goto_statement_dispose (GObject * obj);


ValaCCodeGotoStatement* vala_ccode_goto_statement_new (const char* name) {
	GParameter * __params;
	GParameter * __params_it;
	ValaCCodeGotoStatement * self;
	g_return_val_if_fail (name != NULL, NULL);
	__params = g_new0 (GParameter, 1);
	__params_it = __params;
	(__params_it->name = "name", g_value_init (&__params_it->value, G_TYPE_STRING), g_value_set_string (&__params_it->value, name), __params_it++);
	self = g_object_newv (VALA_TYPE_CCODE_GOTO_STATEMENT, __params_it - __params, __params);
	while (__params_it > __params) {
		--__params_it;
		g_value_unset (&__params_it->value);
	}
	g_free (__params);
	return self;
}


static void vala_ccode_goto_statement_real_write (ValaCCodeNode* base, ValaCCodeWriter* writer) {
	ValaCCodeGotoStatement * self;
	self = VALA_CCODE_GOTO_STATEMENT (base);
	g_return_if_fail (VALA_IS_CCODE_WRITER (writer));
	vala_ccode_writer_write_indent (writer, NULL);
	vala_ccode_writer_write_string (writer, "goto ");
	vala_ccode_writer_write_string (writer, vala_ccode_goto_statement_get_name (self));
	vala_ccode_writer_write_string (writer, ";");
	vala_ccode_writer_write_newline (writer);
}


char* vala_ccode_goto_statement_get_name (ValaCCodeGotoStatement* self) {
	g_return_val_if_fail (VALA_IS_CCODE_GOTO_STATEMENT (self), NULL);
	return self->priv->_name;
}


void vala_ccode_goto_statement_set_name (ValaCCodeGotoStatement* self, const char* value) {
	char* _tmp1;
	g_return_if_fail (VALA_IS_CCODE_GOTO_STATEMENT (self));
	_tmp1 = NULL;
	self->priv->_name = (_tmp1 = g_strdup (value), (self->priv->_name = (g_free (self->priv->_name), NULL)), _tmp1);
}


static void vala_ccode_goto_statement_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	ValaCCodeGotoStatement * self;
	self = VALA_CCODE_GOTO_STATEMENT (object);
	switch (property_id) {
		case VALA_CCODE_GOTO_STATEMENT_NAME:
		g_value_set_string (value, vala_ccode_goto_statement_get_name (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void vala_ccode_goto_statement_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	ValaCCodeGotoStatement * self;
	self = VALA_CCODE_GOTO_STATEMENT (object);
	switch (property_id) {
		case VALA_CCODE_GOTO_STATEMENT_NAME:
		vala_ccode_goto_statement_set_name (self, g_value_get_string (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void vala_ccode_goto_statement_class_init (ValaCCodeGotoStatementClass * klass) {
	vala_ccode_goto_statement_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (ValaCCodeGotoStatementPrivate));
	G_OBJECT_CLASS (klass)->get_property = vala_ccode_goto_statement_get_property;
	G_OBJECT_CLASS (klass)->set_property = vala_ccode_goto_statement_set_property;
	G_OBJECT_CLASS (klass)->dispose = vala_ccode_goto_statement_dispose;
	VALA_CCODE_NODE_CLASS (klass)->write = vala_ccode_goto_statement_real_write;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_CCODE_GOTO_STATEMENT_NAME, g_param_spec_string ("name", "name", "name", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT));
}


static void vala_ccode_goto_statement_init (ValaCCodeGotoStatement * self) {
	self->priv = VALA_CCODE_GOTO_STATEMENT_GET_PRIVATE (self);
}


static void vala_ccode_goto_statement_dispose (GObject * obj) {
	ValaCCodeGotoStatement * self;
	self = VALA_CCODE_GOTO_STATEMENT (obj);
	(self->priv->_name = (g_free (self->priv->_name), NULL));
	G_OBJECT_CLASS (vala_ccode_goto_statement_parent_class)->dispose (obj);
}


GType vala_ccode_goto_statement_get_type (void) {
	static GType vala_ccode_goto_statement_type_id = 0;
	if (G_UNLIKELY (vala_ccode_goto_statement_type_id == 0)) {
		static const GTypeInfo g_define_type_info = { sizeof (ValaCCodeGotoStatementClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_ccode_goto_statement_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaCCodeGotoStatement), 0, (GInstanceInitFunc) vala_ccode_goto_statement_init };
		vala_ccode_goto_statement_type_id = g_type_register_static (VALA_TYPE_CCODE_STATEMENT, "ValaCCodeGotoStatement", &g_define_type_info, 0);
	}
	return vala_ccode_goto_statement_type_id;
}




